@@ -19,6 +19,9 @@
 # THE SOFTWARE.
 
 import logging
+import pathlib
+import tomllib
+
 from configparser import ConfigParser
 from datetime import datetime
 from os import access, chmod, environ, listdir, makedirs, walk, X_OK
@@ -67,7 +70,7 @@ def _copy_static_files(src_dir, debian_dir):
                 copy(join(root, fn), join(dst_dir, fn))
 
 
-def debianize(dpath, ctx, profile=None):
+async def debianize(dpath, ctx, profile=None):
     update_ctx(dpath, ctx)
 
     setupcfg_fpath = join(dpath, 'setup.cfg')
@@ -119,7 +122,7 @@ def debianize(dpath, ctx, profile=None):
         fpath = abspath(join(o_dpath, 'hooks', 'pre'))
         if exists(fpath) and access(fpath, X_OK):
             _dump_ctx(ctx)
-            code = yield from execute([fpath, ctx['src_name'],
+            code = await execute([fpath, ctx['src_name'],
                                        ctx['version'], ctx['debian_revision']],
                                       cwd=dpath)
             if code != 0:
@@ -147,7 +150,7 @@ def debianize(dpath, ctx, profile=None):
     control(dpath, ctx, env)
     rules(dpath, ctx, env)
     chmod(join(dpath, 'debian', 'rules'), 0o755)
-    initial_release = yield from changelog(dpath, ctx, env)
+    initial_release = await changelog(dpath, ctx, env)
     if initial_release:
         itp_mail(dpath, ctx, env)
     copyright(dpath, ctx, env)
@@ -159,7 +162,7 @@ def debianize(dpath, ctx, profile=None):
         fpath = join(o_dpath, 'hooks', 'post')
         if exists(fpath) and access(fpath, X_OK):
             _dump_ctx(ctx)
-            code = yield from execute([fpath, ctx['src_name'],
+            code = await execute([fpath, ctx['src_name'],
                                        ctx['version'], ctx['debian_revision'],
                                        VERSION],
                                       cwd=dpath)
@@ -311,7 +314,26 @@ def control(dpath, ctx, env):
                 except Exception as err:
                     log.warn('cannot parse build dependency: %s', err)
 
-    if exists(join(dpath, 'setup.py')):
+    pyproject_file = pathlib.Path(dpath, 'pyproject.toml')
+    if pyproject_file.exists():
+        ctx['pybuild_depends'] = 'pybuild-plugin-pyproject'
+        pyproject_toml = tomllib.loads(pyproject_file.read_text())
+        # static list of build backends and what package to install to use them
+        backends = {
+            'flit': 'flit',
+            'hatchling': 'python3-hatchling',
+            'mesonpy': 'python3-mesonpy',
+            'pdm': 'python3-pdm-pep517',
+            'poetry': 'python3-poetry-core',
+            'setuptools': 'python3-setuptools',
+            'sipbuild': 'python3-sipbuild',
+        }
+        for _backend, _package in backends.items():
+            if _backend in pyproject_toml['build-system']['build-backend']:
+                ctx['pybuild_depends'] += f', {_package}'
+
+    elif exists(join(dpath, 'setup.py')):
+        ctx['pybuild_depends'] = 'dh-python'
         with open(join(dpath, 'setup.py')) as fp:
             for line in fp:
                 if line.startswith('#'):
@@ -350,7 +372,7 @@ def rules(dpath, ctx, env):
     return ctx
 
 
-def changelog(dpath, ctx, env):
+async def changelog(dpath, ctx, env):
     change = ctx.get('message', f"Initial release, autogenerated by py2dsp/{VERSION}; Closes: #NNNNNN")
     version = ctx['debian_version']
     distribution = ctx.get('distribution', 'UNRELEASED')
@@ -363,7 +385,7 @@ def changelog(dpath, ctx, env):
                 log.debug('changelog doesn\'t need an update')
                 return
             else:
-                yield from execute(['dch', '--force-distribution', '--distribution', distribution,
+                await execute(['dch', '--force-distribution', '--distribution', distribution,
                                     '--newversion', version, '-m', change], cwd=dpath)
         return
 
