@@ -18,9 +18,7 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
-import atexit
 import logging
-import asyncio
 from os import environ
 from os.path import exists, join
 from xmlrpc.client import ServerProxy
@@ -35,28 +33,24 @@ PYPI_XMLRPC_URL = environ.get('PYPI_XMLRPC_URL', 'https://pypi.org/pypi')
 log = logging.getLogger('pypi2deb')
 
 
-@asyncio.coroutine
-def get_pypi_info(name, version=None):
+async def get_pypi_info(name, version=None):
     url = PYPI_JSON_URL + '/' + name
     if version:
         url += '/' + version
     url += '/json'
-    session = None
-    try:
-        session = aiohttp.ClientSession()
-        response = yield from session.get(url)
-    except Exception as err:
-        log.error('invalid project name: {} ({})'.format(name, err))
-    else:
+
+    async with aiohttp.ClientSession() as session:
         try:
-            result = yield from response.json()
+            response = await session.get(url)
+        except Exception as err:
+            log.error('invalid project name: {} ({})'.format(name, err))
+            return
+        try:
+            result = await response.json()
         except Exception as err:
             log.warn('cannot download %s %s details from PyPI: %r', name, version, err)
             return
         return result
-    finally:
-        if session is not None:
-            yield from session.close()
 
 
 def parse_pypi_info(data):
@@ -112,55 +106,62 @@ def parse_pkg_info(fpath):
     raise NotImplementedError()  # FIXME
 
 
-@asyncio.coroutine
-def download(name, version=None, destdir='.'):
-    details = yield from get_pypi_info(name, version)
+async def download(name, version=None, destdir='.'):
+    details = await get_pypi_info(name, version)
     if not details:
         raise Exception('cannot get PyPI project details for {}'.format(name))
 
-    if not version:
-        version = details['info']['version']
+    download_version = version or details['info']['version']
 
-    release = details['releases'].get(version, {})
-    if not release:
-        log.debug('missing release of %s %s on PyPI', name, version)
+    package_urls = details['urls']
+    if not package_urls:
+        log.debug('missing release of %s %s on PyPI', name, download_version)
         raise Exception('missing release')
+
     try:
-        release = next((i for i in release if i['python_version'] == 'source'))
+        release = next((i for i in package_urls if i['python_version'] == 'source'))
     except StopIteration:
         release = None
 
     if not release:
-        raise Exception('source package not available on PyPI')
+        available_files = ", ".join("{} (package type: {})".format(f["filename"], f["packagetype"]) for f in package_urls)
+        other_versions_text = ""
+        if not version and "releases" in details:
+            other_available_versions = [v for v in details["releases"] if v != download_version]
+            if other_available_versions:
+                other_versions_text = " Some other versions found for {}: {}{}.".format(
+                    name,
+                ", ".join(other_available_versions[:5]),
+                " and others" if len(other_available_versions) > 5 else ""
+                )
+            else:
+                other_versions_text = " No other versions for {} found.".format(name)
+        raise Exception('Source package for {} version {} not available on PyPI. Available files: {}.{}'.format(name, download_version, available_files, other_versions_text))
 
-    orig_ext = ext = release['filename'].replace('{}-{}.'.format(name, version), '')
+    orig_ext = ext = release['filename'].replace('{}-{}.'.format(name, download_version), '')
     if ext not in {'tar.gz', 'tar.bz2', 'tar.xz'}:
         ext = 'tar.xz'
 
-    fname = '{}_{}.orig.{}'.format(pkg_name(name), version, ext)
+    fname = '{}_{}.orig.{}'.format(pkg_name(name), download_version, ext)
 
     fpath = join(destdir, fname)
     if exists(fpath):
         return fname
 
-    session = None
-    try:
-        session = aiohttp.ClientSession()
-        response = yield from session.get(release['url'])
+    async with aiohttp.ClientSession() as session:
+        log.debug(f"fetching upstream tarball from {release['url']}")
+        response = await session.get(release['url'])
         with open(fpath if ext == orig_ext else join(destdir, release['filename']), 'wb') as fp:
-            data = yield from response.read()
+            data = await response.read()
             fp.write(data)
-    finally:
-        if session is not None:
-            yield from session.close()
 
     if orig_ext != ext:
         cmd = ['mk-origtargz', '--rename', '--compression', 'xz',
-               '--package', pkg_name(details['info']['name']), '--version', version,
+               '--package', pkg_name(details['info']['name']), '--version', download_version,
                '--directory', destdir,
                '--repack', join(destdir, release['filename'])]
         # TODO: add --copyright-file if overriden copyright file is available
-        yield from execute(cmd)
+        await execute(cmd)
 
     return fname
 
@@ -169,11 +170,11 @@ def download(name, version=None, destdir='.'):
 def list_packages(classifiers=None):
     client = ServerProxy(PYPI_XMLRPC_URL)
     if not classifiers:
-        # packages = yield from client.list_packages()
+        # packages = await client.list_packages()
         packages = client.list_packages()
         packages = dict.fromkeys(packages)
     else:
-        # packages = yield from client.browse(classifiers)
+        # packages = await client.browse(classifiers)
         packages = client.browse(classifiers)
         # browse returns all versions, use the latest one only
         # TODO: use LooseVersion
